/**
 * ShapeChange - processing application schemas for geographic information
 *
 * This file is part of ShapeChange. ShapeChange takes a ISO 19109 
 * Application Schema from a UML model and translates it into a 
 * GML Application Schema or other implementation representations.
 *
 * Additional information about the software can be found at
 * http://shapechange.net/
 *
 * (c) 2002-2013 interactive instruments GmbH, Bonn, Germany
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact:
 * interactive instruments GmbH
 * Trierer Strasse 70-72
 * 53115 Bonn
 * Germany
 */
package de.interactive_instruments.ShapeChange.Model.Generic;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import de.interactive_instruments.ShapeChange.Options;
import de.interactive_instruments.ShapeChange.ShapeChangeResult;
import de.interactive_instruments.ShapeChange.Model.AssociationInfoImpl;
import de.interactive_instruments.ShapeChange.Model.ClassInfo;
import de.interactive_instruments.ShapeChange.Model.Model;
import de.interactive_instruments.ShapeChange.Model.PropertyInfo;

/**
 * @author echterhoff
 * 
 */
public class GenericAssociationInfo extends AssociationInfoImpl {

	protected Options options = null;
	protected ShapeChangeResult result = null;
	protected GenericModel model = null;
	protected String id = null;
	protected String name = null;
	protected HashSet<String> stereotypes = null;
	protected HashMap<String, String> taggedValues = null;

	protected PropertyInfo end1 = null;
	protected PropertyInfo end2 = null;
	protected ClassInfo assocClass = null;

	/**
	 * @param end1
	 *            the end1 to set
	 */
	public void setEnd1(PropertyInfo end1) {
		this.end1 = end1;
	}

	/**
	 * @param end2
	 *            the end2 to set
	 */
	public void setEnd2(PropertyInfo end2) {
		this.end2 = end2;
	}

	/**
	 * @param assocClass
	 *            the assocClass to set
	 */
	public void setAssocClass(ClassInfo assocClass) {
		this.assocClass = assocClass;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#stereotypes()
	 */
	public Set<String> stereotypes() {
		// Return copy of cache
		Set<String> res = (stereotypes==null? new HashSet<String>() : new HashSet<String>(stereotypes));
		return res;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.InfoImpl#aliasName()
	 */
	@Override
	public String aliasName() {
		return aliasName;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.InfoImpl#documentation()
	 */
	@Override
	public String documentation() {
		return documentation;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#id()
	 */
	public String id() {
		return id;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#model()
	 */
	public Model model() {
		return model;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#name()
	 */
	public String name() {
		return name;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#options()
	 */
	public Options options() {
		return options;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#result()
	 */
	public ShapeChangeResult result() {
		return result;
	}

	/**
	 * @param aliasName
	 */
	public void setAliasName(String aliasName) {
		this.aliasName = aliasName;

	}

	/**
	 * @param documentation
	 */
	public void setDocumentation(String documentation) {
		this.documentation = documentation;

		// now invalidate the definition and description values
		this.definition = null;
		this.description = null;
	}

	/**
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;

	}

	/**
	 * @param model
	 */
	public void setModel(GenericModel model) {
		this.model = model;

	}

	/**
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;

	}

	/**
	 * @param options
	 */
	public void setOptions(Options options) {
		this.options = options;
	}

	/**
	 * @param result
	 */
	public void setResult(ShapeChangeResult result) {
		this.result = result;

	}

	/**
	 * @param stereotypeSet
	 */
	public void setStereotypes(Set<String> stereotypeSet) {
		stereotypes = new HashSet<String>();
		if (stereotypeSet!=null) {
			for (String st : stereotypeSet) {
				this.stereotypes.add(options.normalizeStereotype(st));
			}
		}
	}

	/**
	 * @param taggedValues
	 */
	public void setTaggedValues(HashMap<String, String> taggedValues) {
		this.taggedValues = taggedValues;

	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#taggedValue(java.lang.String)
	 */
	public String taggedValue(String tag) {
		return taggedValues.get(tag);
	}

	/**
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.Info#taggedValues()
	 */
	public HashMap<String, String> taggedValues() {
		return taggedValues;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#taggedValues(java.lang.String)
	 */
	public HashMap<String, String> taggedValues(String tagList) {
		// Split and allocate for result
		String[] tags = tagList.split(",");
		HashMap<String, String> res = new HashMap<String, String>(tags.length);
		// Retrieve the requested tags and fill the output map
		for (String tag : tags) {
			String t = tag.trim();
			String v = taggedValues.get(t);
			if (v != null) {
				res.put(t, v);
			}
		}
		// Return constructed result
		return res;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.AssociationInfo#end1()
	 */
	public PropertyInfo end1() {
		return end1;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.AssociationInfo#end2()
	 */
	public PropertyInfo end2() {
		return end2;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.AssociationInfo#assocClass()
	 */
	public ClassInfo assocClass() {
		return assocClass;
	}
	
	

}
