/**
 * ShapeChange - processing application schemas for geographic information
 *
 * This file is part of ShapeChange. ShapeChange takes a ISO 19109 
 * Application Schema from a UML model and translates it into a 
 * GML Application Schema or other implementation representations.
 *
 * Additional information about the software can be found at
 * http://shapechange.net/
 *
 * (c) 2002-2013 interactive instruments GmbH, Bonn, Germany
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact:
 * interactive instruments GmbH
 * Trierer Strasse 70-72
 * 53115 Bonn
 * Germany
 */
package de.interactive_instruments.ShapeChange.Model.Generic;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;

import de.interactive_instruments.ShapeChange.MessageSource;
import de.interactive_instruments.ShapeChange.Multiplicity;
import de.interactive_instruments.ShapeChange.Options;
import de.interactive_instruments.ShapeChange.ShapeChangeResult;
import de.interactive_instruments.ShapeChange.StructuredNumber;
import de.interactive_instruments.ShapeChange.Type;
import de.interactive_instruments.ShapeChange.Model.AssociationInfo;
import de.interactive_instruments.ShapeChange.Model.ClassInfo;
import de.interactive_instruments.ShapeChange.Model.Constraint;
import de.interactive_instruments.ShapeChange.Model.Model;
import de.interactive_instruments.ShapeChange.Model.PropertyInfo;
import de.interactive_instruments.ShapeChange.Model.PropertyInfoImpl;

/**
 * 
 */
public class GenericPropertyInfo extends PropertyInfoImpl implements
		MessageSource {

	protected Options options = null;
	protected ShapeChangeResult result = null;
	protected GenericModel model = null;
	protected String id = null;
	protected String name = null;
	protected HashSet<String> stereotypes = null;
	protected HashMap<String, String> taggedValues = null;

	protected boolean isDerived = false;
	protected boolean isReadOnly = false;
	protected boolean isAttribute = true;
	protected Type typeInfo = null;
	protected boolean isNavigable = true;
	protected boolean isOrdered = false;
	protected boolean isComposition = false;
	protected boolean isAggregation = false;
	protected Multiplicity cardinality = new Multiplicity("1");
	protected String initialValue = null;
	protected String inlineOrByReference = "inlineOrByReference";
	protected String defaultCodeSpace = "";
	protected boolean isMetadata = false;
	protected PropertyInfo reverseProperty = null;
	protected ClassInfo inClass = null;
	protected StructuredNumber sequenceNumber = null;
	protected boolean implementedByNilReason = false;
	protected boolean voidable = false;
	protected Vector<Constraint> constraints = new Vector<Constraint>();
	protected AssociationInfo association = null;
	protected int categoryOfValue = Options.UNKNOWN;

	public GenericPropertyInfo(GenericModel model, String id, String name,
			int categoryOfValue) {

		this.model = model;
		this.options = model.options();
		this.result = model.result();
		this.id = id;
		this.name = name;
		this.categoryOfValue = categoryOfValue;
	}

	/**
	 * @return the isMetadata
	 */
	public boolean isMetadata() {
		return isMetadata;
	}

	/**
	 * @param isAttribute
	 *            the isAttribute to set
	 */
	public void setAttribute(boolean isAttribute) {
		this.isAttribute = isAttribute;
	}

	/**
	 * @param isMetadata
	 *            the isMetadata to set
	 */
	public void setMetadata(boolean isMetadata) {
		this.isMetadata = isMetadata;
	}

	/**
	 * @param reverseProperty
	 *            the reverseProperty to set
	 */
	public void setReverseProperty(PropertyInfo reverseProperty) {
		this.reverseProperty = reverseProperty;
	}

	/**
	 * @param inClass
	 *            the inClass to set
	 */
	public void setInClass(ClassInfo inClass) {
		this.inClass = inClass;
	}

	/**
	 * @param implementedByNilReason
	 *            the implementedByNilReason to set
	 */
	public void setImplementedByNilReason(boolean implementedByNilReason) {
		this.implementedByNilReason = implementedByNilReason;
	}

	/**
	 * @param association
	 *            the association to set
	 */
	public void setAssociation(AssociationInfo association) {
		this.association = association;
	}

	/**
	 * @param categoryOfValue
	 *            the categoryOfValue to set
	 */
	public void setCategoryOfValue(int categoryOfValue) {
		this.categoryOfValue = categoryOfValue;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#isDerived()
	 */
	public boolean isDerived() {
		return isDerived;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#isAttribute()
	 */
	public boolean isAttribute() {
		return isAttribute;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#typeInfo()
	 */
	public Type typeInfo() {
		return typeInfo;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#isNavigable()
	 */
	public boolean isNavigable() {
		return isNavigable;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#isOrdered()
	 */
	public boolean isOrdered() {
		return isOrdered;
	}

	public boolean hasConstraints() {
		if (this.constraints == null || this.constraints.isEmpty()) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#isComposition()
	 */
	public boolean isComposition() {
		return isComposition;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#isAggregation()
	 */
	public boolean isAggregation() {
		return isAggregation;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#cardinality()
	 */
	public Multiplicity cardinality() {
		return cardinality;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#initialValue()
	 */
	public String initialValue() {
		return initialValue;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#inlineOrByReference
	 *      ()
	 */
	public String inlineOrByReference() {
		return inlineOrByReference;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#defaultCodeSpace
	 *      ()
	 */
	public String defaultCodeSpace() {
		return defaultCodeSpace;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#reverseProperty
	 *      ()
	 */
	public PropertyInfo reverseProperty() {
		return reverseProperty;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#inClass()
	 */
	public ClassInfo inClass() {
		return inClass;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#inClass(de.
	 *      interactive_instruments.ShapeChange.Model.ClassInfo)
	 */
	public void inClass(ClassInfo ci) {
		/*
		 * NOTE for cast: the cast should be safe, at least once the
		 * GenericModel has been created/parsed (because the constructor there
		 * ensures that inClass for GenericPropertyInfos is set to the correct
		 * GenericClassInfos
		 */
		inClass = (GenericClassInfo) ci;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#sequenceNumber
	 *      ()
	 */
	public StructuredNumber sequenceNumber() {
		return sequenceNumber;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#voidable()
	 */
	public boolean voidable() {
		return voidable;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#constraints()
	 */
	public Vector<Constraint> constraints() {
		return constraints;
	}

	/**
	 * 
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfo#association()
	 */
	public AssociationInfo association() {
		return association;
	}

	/**
	 * @param cardinality
	 *            the cardinality to set
	 */
	public void setCardinality(Multiplicity cardinality) {
		this.cardinality = cardinality;
	}

	/**
	 * @param constraints
	 *            the constraints to set
	 */
	public void setConstraints(Vector<Constraint> constraints) {
		this.constraints = constraints;
	}

	/**
	 * @param defaultCodeSpace
	 *            the defaultCodeSpace to set
	 */
	public void setDefaultCodeSpace(String defaultCodeSpace) {
		this.defaultCodeSpace = defaultCodeSpace;
	}

	/**
	 * @param initialValue
	 *            the initialValue to set
	 */
	public void setInitialValue(String initialValue) {
		this.initialValue = initialValue;
	}

	/**
	 * @param inlineOrByReference
	 *            the inlineOrByReference to set
	 */
	public void setInlineOrByReference(String inlineOrByReference) {
		this.inlineOrByReference = inlineOrByReference;
	}

	/**
	 * @param isAggregation
	 *            the isAggregation to set
	 */
	public void setAggregation(boolean isAggregation) {
		this.isAggregation = isAggregation;
	}

	/**
	 * @param isComposition
	 *            the isComposition to set
	 */
	public void setComposition(boolean isComposition) {
		this.isComposition = isComposition;
	}

	/**
	 * @param isDerived
	 *            the isDerived to set
	 */
	public void setDerived(boolean isDerived) {
		this.isDerived = isDerived;
	}

	/**
	 * @param isNavigable
	 *            the isNavigable to set
	 */
	public void setNavigable(boolean isNavigable) {
		this.isNavigable = isNavigable;
	}

	/**
	 * @param isOrdered
	 *            the isOrdered to set
	 */
	public void setOrdered(boolean isOrdered) {
		this.isOrdered = isOrdered;
	}

	/**
	 * @param sequenceNumber
	 *            the sequenceNumber to set
	 */
	public void setSequenceNumber(StructuredNumber sequenceNumber) {
		this.sequenceNumber = sequenceNumber;
	}

	/**
	 * @param typeInfo
	 *            the typeInfo to set
	 */
	public void setTypeInfo(Type typeInfo) {
		this.typeInfo = typeInfo;
	}

	/**
	 * @param voidable
	 *            the voidable to set
	 */
	public void setVoidable(boolean voidable) {
		this.voidable = voidable;
	}

	public void setNilReasonAllowed(boolean nilReasonAllowed) {
		this.nilReasonAllowed = nilReasonAllowed;
	}

	public void setRestriction(boolean isRestriction) {
		this.restriction = isRestriction;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#stereotypes()
	 */
	public Set<String> stereotypes() {
		// Return copy of cache
		Set<String> res = (stereotypes == null ? new HashSet<String>()
				: new HashSet<String>(stereotypes));
		return res;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.InfoImpl#aliasName()
	 */
	@Override
	public String aliasName() {
		return aliasName;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.InfoImpl#documentation()
	 */
	@Override
	public String documentation() {
		return documentation;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#id()
	 */
	public String id() {
		return id;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#model()
	 */
	public Model model() {
		return model;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#name()
	 */
	public String name() {
		return name;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#options()
	 */
	public Options options() {
		return options;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#result()
	 */
	public ShapeChangeResult result() {
		return result;
	}

	/**
	 * @param aliasName
	 */
	public void setAliasName(String aliasName) {
		this.aliasName = aliasName;

	}

	/**
	 * Sets the documentation of this property to the given value. Also
	 * invalidates the definition and description values so that they are
	 * computed based upon the new documentation.
	 * 
	 * @param documentation
	 */
	public void setDocumentation(String documentation) {
		this.documentation = documentation;

		// now invalidate the definition and description values
		this.definition = null;
		this.description = null;
	}

	/**
	 * @param id
	 */
	public void setId(String id) {
		this.id = id;

	}

	/**
	 * @param model
	 */
	public void setModel(GenericModel model) {
		this.model = model;

	}

	/**
	 * @param name
	 */
	public void setName(String name) {
		this.name = name;

	}

	/**
	 * @param options
	 */
	public void setOptions(Options options) {
		this.options = options;
	}

	/**
	 * @param result
	 */
	public void setResult(ShapeChangeResult result) {
		this.result = result;

	}

	/**
	 * @param stereotypeSet
	 */
	public void setStereotypes(Set<String> stereotypeSet) {
		stereotypes = new HashSet<String>();
		if (stereotypeSet != null) {
			for (String st : stereotypeSet) {
				this.stereotypes.add(options.normalizeStereotype(st));
			}
		}
	}

	/**
	 * @param stereotype
	 */
	public void setStereotype(String stereotype) {
		stereotypes = new HashSet<String>();
		if (stereotype != null) {
			this.stereotypes.add(options.normalizeStereotype(stereotype));
		}
	}

	/**
	 * @param taggedValues
	 * @param updateFields
	 *            true if class fields should be updated based upon information
	 *            from given tagged values, else false
	 */
	public void setTaggedValues(HashMap<String, String> taggedValues,
			boolean updateFields) {

		this.taggedValues = taggedValues;

		// Now update fields, if they are affected by tagged values

		if (updateFields && taggedValues != null) {

			for (String key : taggedValues.keySet()) {

				updateFieldsForTaggedValue(key, taggedValues.get(key));
			}
		}
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#taggedValue(java.lang.String)
	 */
	public String taggedValue(String tag) {
		return taggedValues.get(tag);
	}

	/**
	 * 
	 * @see de.interactive_instruments.ShapeChange.Model.Info#taggedValues()
	 */
	public HashMap<String, String> taggedValues() {
		return taggedValues;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.Info#taggedValues(java.lang.String)
	 */
	public HashMap<String, String> taggedValues(String tagList) {
		// Split and allocate for result
		String[] tags = tagList.split(",");
		HashMap<String, String> res = new HashMap<String, String>(tags.length);
		// Retrieve the requested tags and fill the output map
		for (String tag : tags) {
			String t = tag.trim();
			String v = taggedValues.get(t);
			if (v != null) {
				res.put(t, v);
			}
		}
		// Return constructed result
		return res;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfoImpl#implementedByNilReason()
	 */
	@Override
	public boolean implementedByNilReason() {
		return implementedByNilReason;
	}

	/**
	 * @see de.interactive_instruments.ShapeChange.Model.PropertyInfoImpl#categoryOfValue()
	 */
	@Override
	public int categoryOfValue() {
		if (typeInfo != null) {
			ClassInfo ci = model().classById(typeInfo().id);
			if (ci != null)
				return ci.category();
			else
				return categoryOfValue;
		} else
			return categoryOfValue;
	}

	@SuppressWarnings("unchecked")
	public GenericPropertyInfo createCopy(String copyId) {

		GenericPropertyInfo copy = new GenericPropertyInfo(model, copyId, name,
				categoryOfValue);

		copy.setAliasName(aliasName);
		copy.setDocumentation(documentation);
		copy.setStereotypes(stereotypes);

		HashMap<String, String> tvcopy = new HashMap<String, String>(
				taggedValues.size());
		for (String key : taggedValues.keySet()) {
			tvcopy.put(key, taggedValues.get(key));
		}
		copy.setTaggedValues(tvcopy, false);
		// copy.setTaggedValues((HashMap<String, String>) taggedValues.clone(),
		// false);

		copy.setDerived(isDerived);
		copy.setReadOnly(isReadOnly);
		copy.setAttribute(isAttribute);
		Type t = new Type();
		t.id = typeInfo.id;
		t.name = typeInfo.name;
		copy.setTypeInfo(t);
		copy.setNavigable(isNavigable);
		copy.setOrdered(isOrdered);
		copy.setComposition(isComposition);
		copy.setAggregation(isAggregation);
		copy.setCardinality(new Multiplicity(cardinality.toString()));
		copy.setInitialValue(initialValue);
		copy.setInlineOrByReference(inlineOrByReference);
		copy.setDefaultCodeSpace(defaultCodeSpace);
		copy.setMetadata(isMetadata);
		copy.setReverseProperty(reverseProperty);
		copy.setInClass(inClass);

		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < sequenceNumber.components.length - 1; i++) {
			sb.append(sequenceNumber.components[i] + ".");
		}
		sb.append(sequenceNumber.components[sequenceNumber.components.length - 1]);
		StructuredNumber res = new StructuredNumber(sb.toString());

		copy.setSequenceNumber(res);
		copy.setImplementedByNilReason(implementedByNilReason);
		copy.setVoidable(voidable);
		copy.setConstraints(constraints);
		copy.setAssociation(association);
		copy.setRestriction(restriction);
		copy.setNilReasonAllowed(nilReasonAllowed);

		return copy;
	}

	public String toString(String indent) {

		StringBuffer sb = new StringBuffer();

		sb.append(indent + name + "\n");
		sb.append(indent + indent + "tagged values: ");
		for (String key : taggedValues.keySet()) {
			sb.append("(" + key + "," + taggedValues.get(key) + ") ");
		}
		sb.append("\r\n");

		return sb.toString();
	}

	public String message(int mnr) {

		switch (mnr) {
		case 1:
			return "(GenericPropertyInfo) When setting tagged value '$1$', a boolean value (either 'false' or 'true') was expected. Found '$2$' - cannot set field(s) for this tagged value.";
		case 2:
			return "(GenericPropertyInfo) When setting tagged value '$1$', one of the values 'inline', 'byReference', or 'inlineOrByReference' was expected. Found '$2$' - cannot set field for this tagged value.";

		default:
			return "(Unknown message)";
		}
	}

	/**
	 * @return the isReadOnly
	 */
	public boolean isReadOnly() {
		return isReadOnly;
	}

	/**
	 * @param isReadOnly
	 *            the isReadOnly to set
	 */
	public void setReadOnly(boolean isReadOnly) {
		this.isReadOnly = isReadOnly;
	}

	/**
	 * @param tvName
	 * @param tvValue
	 * @param updateFields
	 *            true if property fields should be updated based upon
	 *            information from given tagged value, else false
	 */
	public void setTaggedValue(String tvName, String tvValue,
			boolean updateFields) {

		if (taggedValues == null) {
			taggedValues = new HashMap<String, String>();
		}

		this.taggedValues.put(tvName, tvValue);

		if (updateFields) {
			updateFieldsForTaggedValue(tvName, tvValue);
		}
	}

	/**
	 * Encapsulates the logic to update property fields based upon the value of
	 * a named tagged value.
	 * 
	 * @param taggedValueName
	 * @param taggedValueValue
	 */
	private void updateFieldsForTaggedValue(String tvName, String tvValue) {

		// TODO add more updates for relevant tagged values

		if (tvName.equalsIgnoreCase("isMetadata")) {

			if (tvValue.equalsIgnoreCase("true")) {
				this.setMetadata(true);
			} else if (tvValue.equalsIgnoreCase("false")) {
				this.setMetadata(false);
			} else {
				result.addWarning(this, 1, tvName, tvValue);
			}
		} else if (tvName.equalsIgnoreCase("gmlImplementedByNilReason")) {

			if (tvValue.equalsIgnoreCase("true")) {
				this.setImplementedByNilReason(true);
			} else if (tvValue.equalsIgnoreCase("false")) {
				this.setImplementedByNilReason(false);
			} else {
				result.addWarning(this, 1, tvName, tvValue);
			}
		} else if (tvName.equalsIgnoreCase("nilReasonAllowed")) {

			if (tvValue.equalsIgnoreCase("true")) {
				this.setNilReasonAllowed(true);
			} else if (tvValue.equalsIgnoreCase("false")) {
				this.setNilReasonAllowed(false);
			} else {
				result.addWarning(this, 1, tvName, tvValue);
			}
		} else if (tvName.equalsIgnoreCase("inlineOrByReference")) {

			if (tvValue.equalsIgnoreCase("inline")) {
				this.setInlineOrByReference("inline");
			} else if (tvValue.equalsIgnoreCase("byReference")) {
				this.setInlineOrByReference("byReference");
			} else if (tvValue.equalsIgnoreCase("inlineOrByReference")) {
				this.setInlineOrByReference("inlineOrByReference");
			} else {
				result.addWarning(this, 2, tvName, tvValue);
			}
		} else if (tvName.equalsIgnoreCase("alias")) {

			this.setAliasName(tvValue);

		} else if (tvName.equalsIgnoreCase("documentation")) {

			this.setDocumentation(tvValue);
		}
	}

}
