/**
 * ShapeChange - processing application schemas for geographic information
 *
 * This file is part of ShapeChange. ShapeChange takes a ISO 19109 
 * Application Schema from a UML model and translates it into a 
 * GML Application Schema or other implementation representations.
 *
 * Additional information about the software can be found at
 * http://shapechange.net/
 *
 * (c) 2002-2012 interactive instruments GmbH, Bonn, Germany
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact:
 * interactive instruments GmbH
 * Trierer Strasse 70-72
 * 53115 Bonn
 * Germany
 */

package de.interactive_instruments.ShapeChange.Model;

import java.util.HashMap;
import java.util.Set;

import de.interactive_instruments.ShapeChange.Options;
import de.interactive_instruments.ShapeChange.ShapeChangeResult;

public interface Info {
	public Options options();

	public ShapeChangeResult result();

	public Model model();

	public String id();

	public String name();

	public String aliasName();

	/**
	 * Fully qualified UML name.
	 */
	public String fullName();

	public String documentation();

	public String definition();

	public String description();

	public Set<String> stereotypes();

	/**
	 * Test whether the element carries a certain stereotype
	 */
	public boolean stereotype(String stereotype);

	/**
	 * @param tag
	 *            name of the tagged value to look up
	 * @return the tagged value for the tag given or
	 *         <code>null<code> if the tagged value is missing.
	 */
	public String taggedValue(String tag);

	/**
	 * @return a map with the tagged values defined for this object (key: tagged
	 *         value name, value: the value of the tagged value); can be an
	 *         empty map but not <code>null</code>
	 */
	public HashMap<String, String> taggedValues();

	/**
	 * Return all the tagged values listed in the input string. Tags are assumed
	 * to be separated by commas.
	 */
	public HashMap<String, String> taggedValues(String tagList);

	/**
	 * Identifies the encoding rule relevant on the element, given the platform.
	 * 
	 * @param platform
	 * @return the encoding rule relevant on the element
	 */
	public String encodingRule(String platform);

	public boolean matches(String rule);

	public void postprocessAfterLoadingAndValidate();

}