/**
 * ShapeChange - processing application schemas for geographic information
 *
 * This file is part of ShapeChange. ShapeChange takes a ISO 19109 
 * Application Schema from a UML model and translates it into a 
 * GML Application Schema or other implementation representations.
 *
 * Additional information about the software can be found at
 * http://shapechange.net/
 *
 * (c) 2002-2012 interactive instruments GmbH, Bonn, Germany
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact:
 * interactive instruments GmbH
 * Trierer Strasse 70-72
 * 53115 Bonn
 * Germany
 */

package de.interactive_instruments.ShapeChange.Model;

import java.util.HashSet;

import de.interactive_instruments.ShapeChange.Options;
import de.interactive_instruments.ShapeChange.ShapeChangeAbortException;
import de.interactive_instruments.ShapeChange.ShapeChangeResult;

public interface Model {
	public Options options();

	public ShapeChangeResult result();

	public void initialise(ShapeChangeResult r, Options o,
			String repositoryFileNameOrConnectionString) throws ShapeChangeAbortException;

	public void initialise(ShapeChangeResult r, Options o,
			String repositoryFileNameOrConnectionString, String username, String password) throws ShapeChangeAbortException;
	
	public int type();

	/**
	 * Collect and return all PackageInfo objects tagged as being a schema. If a
	 * name is given, only the package with the specified name will be
	 * considered.
	 */
	public HashSet<PackageInfo> schemas(String name);

	/**
	 * Return all schemas that are selected using the relevant parameters:
	 * appSchemaName, appSchemaNameRegex, appSchemaNamespaceRegex
	 * 
	 * @see PackageInfo#isSchema()
	 */
	public HashSet<PackageInfo> selectedSchemas();

	/**
	 * Return all ClassInfo objects contained in the given package and in sub-
	 * packages, which belong to the same targetNamespace as the given package.
	 */
	public HashSet<ClassInfo> classes(PackageInfo pi);

	public void postprocessAfterLoadingAndValidate();

	public PackageInfo packageById(String id);

	public ClassInfo classById(String id);

	public ClassInfo classByName(String nam);

	public String characterEncoding();

	public void shutdown();
	
	/**
	 * 
	 * @return all {@link PackageInfo} objects contained in the model
	 */
	public HashSet<PackageInfo> packages();
}
