/**
 * ShapeChange - processing application schemas for geographic information
 *
 * This file is part of ShapeChange. ShapeChange takes a ISO 19109 
 * Application Schema from a UML model and translates it into a 
 * GML Application Schema or other implementation representations.
 *
 * Additional information about the software can be found at
 * http://shapechange.net/
 *
 * (c) 2002-2012 interactive instruments GmbH, Bonn, Germany
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact:
 * interactive instruments GmbH
 * Trierer Strasse 70-72
 * 53115 Bonn
 * Germany
 */

package de.interactive_instruments.ShapeChange.Model;

import java.util.Vector;

import de.interactive_instruments.ShapeChange.Multiplicity;
import de.interactive_instruments.ShapeChange.StructuredNumber;
import de.interactive_instruments.ShapeChange.Type;

public interface PropertyInfo extends Info {

	/**
	 * Find out whether this property is a derived one.
	 * */
	public boolean isDerived();

	/**
	 * Indicate whether this property is an attribute (and not a role)
	 * */
	public boolean isAttribute();

	public Type typeInfo();

	/**
	 * Indicate whether the property is navigable. An attribute is always
	 * navigable, while an association role may not be navigable.
	 * */
	public boolean isNavigable();

	/**
	 * Find out if the property represents an ordered collection.
	 * */
	public boolean isOrdered();

	/**
	 * @return true if the attribute may not be written to after initialization,
	 *         else false (default).
	 */
	public boolean isReadOnly();

	/**
	 * Find out whether this property is a composition.
	 * */
	public boolean isComposition();

	/**
	 * Find out whether this property is an aggregation.
	 * */
	public boolean isAggregation();

	public Multiplicity cardinality();

	/**
	 * @return the initial value of the property in case such a thing is
	 *         specified in the model, null otherwise. This works only for
	 *         attributes.
	 */
	public String initialValue();

	public boolean isRestriction();

	public String inlineOrByReference();

	public String defaultCodeSpace();

	public boolean isMetadata();

	/**
	 * @return the property on the other end of the association;
	 *         <code>null</code> for attribute properties.
	 */
	public PropertyInfo reverseProperty();

	public ClassInfo inClass();

	public void inClass(ClassInfo ci);

	public String qname();

	public StructuredNumber sequenceNumber();

	public boolean implementedByNilReason();

	public boolean nilReasonAllowed();

	public void nilReasonAllowed(boolean b);

	public boolean voidable();

	public Vector<Constraint> constraints();

	public AssociationInfo association();

	public int categoryOfValue();
	
	public Qualifier qualifier(String name);

	public Vector<Qualifier> qualifiers();
};
