package de.interactive_instruments.ShapeChange.Target.EA;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.TimeZone;
import java.util.Map.Entry;
import java.util.Set;

import org.sparx.AttributeTag;
import org.sparx.Collection;
import org.sparx.Connector;
import org.sparx.ConnectorEnd;
import org.sparx.ConnectorTag;
import org.sparx.CreateModelType;
import org.sparx.Element;
import org.sparx.Repository;
import org.sparx.Package;
import org.sparx.RoleTag;
import org.sparx.TaggedValue;

import de.interactive_instruments.ShapeChange.Multiplicity;
import de.interactive_instruments.ShapeChange.Options;
import de.interactive_instruments.ShapeChange.ShapeChangeAbortException;
import de.interactive_instruments.ShapeChange.ShapeChangeResult;
import de.interactive_instruments.ShapeChange.Model.AssociationInfo;
import de.interactive_instruments.ShapeChange.Model.ClassInfo;
import de.interactive_instruments.ShapeChange.Model.Constraint;
import de.interactive_instruments.ShapeChange.Model.Info;
import de.interactive_instruments.ShapeChange.Model.Model;
import de.interactive_instruments.ShapeChange.Model.PackageInfo;
import de.interactive_instruments.ShapeChange.Model.PropertyInfo;
import de.interactive_instruments.ShapeChange.Target.SingleTarget;

public class UmlModel implements SingleTarget {

	public static final int TARGET_UmlModel = 67;

	private static boolean initialised = false;
	private static String outputFilename = null;
	private static Repository rep = null; 
	private static Package pOut = null;
	private static Set<AssociationInfo> associations = new HashSet<AssociationInfo>();
	private static Map<ClassInfo,Element> classes = new HashMap<ClassInfo,Element>();
	private static Map<ClassInfo,ClassInfo> generalisations = new HashMap<ClassInfo,ClassInfo>();
	
	private PackageInfo pi = null;
	private Model model = null;
	private Options options = null;
	private ShapeChangeResult result = null;
	private Map<String,Package> packages = new HashMap<String,Package>();

	// TODO Unit Test
	
	public void initialise(PackageInfo p, Model m, Options o,
			ShapeChangeResult r, boolean diagOnly)
			throws ShapeChangeAbortException {
		
		pi = p;
		model = m;
		options = o;
		result = r;

		if (!initialised) {
			initialised = true;

			outputFilename = options.parameter(this.getClass().getName(),"modelFilename");
			if (outputFilename == null)
				outputFilename = "ShapeChangeExport.eap";

			/** Make sure repository file exists */
			java.io.File repfile = new java.io.File(outputFilename);
			boolean ex = true;
			
			rep = new Repository();

			if(!repfile.exists()) {
				ex = false;
				if(!outputFilename.toLowerCase().endsWith(".eap")) {
					outputFilename += ".eap";
					repfile = new java.io.File(outputFilename);
					ex = repfile.exists();
				}
			}
			String absname = repfile.getAbsolutePath();
			if(!ex) {
				if (!rep.CreateModel(CreateModelType.cmEAPFromBase, absname, 0)) {
					r.addError(null, 31, absname);
					rep = null;
					return;
				}
			} else {
				/** Connect to EA Repository */
				if(!rep.OpenFile(absname)) {
					String errormsg = rep.GetLastError();
					r.addError(null, 30, errormsg, outputFilename);
					rep = null;
					return;
				}
			}

			rep.RefreshModelView(0);
			
	        Collection<Package> c = rep.GetModels();
	        Package root = c.GetAt((short)0);
	        
	        
		    TimeZone tz = TimeZone.getTimeZone("UTC");
			DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm");
			df.setTimeZone(tz);

			pOut = root.GetPackages().AddNew("ShapeChangeOutput-"+df.format(new Date()), "Class View");
	        if (!pOut.Update()) {
	            result.addError("EA-Fehler: "+pOut.GetLastError());
	        }
		}

		if (rep==null || pOut==null)
			return; // repository not initialised
		
		// export app schema package
		clonePackage(pi, pOut);
	}

	private void clonePackage(PackageInfo pSource, Package container) {
		Package pkg = container.GetPackages().AddNew(pSource.name(), "Nothing");
        if (!pkg.Update()) {
            result.addError("EA-Fehler: "+pkg.GetLastError());
        }
        cloneStandarddItems(pkg.GetElement(), pSource);

        packages.put(pSource.name(), pkg);
        
        for (PackageInfo cpi : pSource.containedPackages()) {
        	clonePackage(cpi,pkg);
        }
	}
	
	public void process(ClassInfo ci) {
    	cloneClass(ci);
	}
	
	private void cloneClass(ClassInfo ci) {
		Package pkg = packages.get(ci.pkg().name());
		if (pkg==null) {
			result.addError("Missing package information for class "+ci.fullName());
			return;
		}
		Element e = pkg.GetElements().AddNew(ci.name(), "Class");
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        classes.put(ci, e);
        cloneStandarddItems(e, ci);
        if (ci.isAbstract()) {
        	e.SetAbstract("true");
            if (!e.Update()) {
                result.addError("EA-Fehler: "+e.GetLastError());
            }
        }
        for (Constraint constr : ci.constraints()) {
        	org.sparx.Constraint ct = e.GetConstraints().AddNew(constr.name(), "OCL");
            if (!ct.Update()) {
            	result.addError("EA-Fehler: "+ct.GetLastError());
            }
        	ct.SetNotes(constr.text());
            ct.Update();               
        }
        for (PropertyInfo propi : ci.properties().values()) {
        	if (propi.isAttribute())
        		cloneAttribute(e, propi);
        	else
        		associations.add(propi.association());
        }
        for (String tid : ci.supertypes()) {
        	ClassInfo cix = model.classById(tid);
        	if (cix==null) {
        		result.addError("Supertype with id "+tid+" of class "+ci.name()+" not found.");
        	} else {
        		generalisations.put(ci, cix);
        	}
        }
        for (String tid : ci.subtypes()) {
        	ClassInfo cix = model.classById(tid);
        	if (cix==null) {
        		result.addError("Subtype with id "+tid+" of class "+ci.name()+" not found.");
        	} else {
        		generalisations.put(cix, ci);
        	}
        }
        if (ci.isAssocClass()!=null) {
        	// TODO
        }
	}
	
	private String stereotypesCSV(Set<String> stereotypes) {
		String res = "";
		if (stereotypes!=null)
			for (String s : stereotypes) {
				res += (res.isEmpty()?"":",");
				res += s;
			}
		return res;
	}
	
	private void cloneAttribute(Element e, PropertyInfo propi) {
		org.sparx.Attribute att = e.GetAttributes().AddNew(propi.name(), "");
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        att.SetStyle(propi.aliasName());    
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        att.SetNotes(propi.documentation());    
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        att.SetStereotype(stereotypesCSV(propi.stereotypes()));    
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        for (Entry<String,String> me : propi.taggedValues().entrySet()) {
        	setTaggedValueAtt(att.GetTaggedValues(),me.getKey(),me.getValue());
        }
        att.SetIsDerived(propi.isDerived());
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        att.SetIsOrdered(propi.isOrdered());
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        att.SetDefault(propi.initialValue());
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        att.SetVisibility("public");
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        Multiplicity m = propi.cardinality();
        att.SetLowerBound(""+m.minOccurs);
        if (m.maxOccurs==Integer.MAX_VALUE)
        	att.SetUpperBound("*");
        else
        	att.SetUpperBound(""+m.maxOccurs);
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
        att.SetType(propi.typeInfo().name);
        if (!att.Update()) {
            result.addError("EA-Fehler: "+att.GetLastError());
        }
	}
	
	private void cloneStandarddItems(Element e, Info i) {
        e.SetAlias(i.aliasName());    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetNotes(i.documentation());    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetStereotype(stereotypesCSV(i.stereotypes()).replace(" ", ""));    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.Refresh();
        for (Entry<String,String> me : i.taggedValues().entrySet()) {
        	setTaggedValue(e.GetTaggedValues(),me.getKey(),me.getValue());
        }
	}
	
	private void setTaggedValue(Collection<org.sparx.TaggedValue> cTV, String n, String v) {
		TaggedValue tv = null;
		for (TaggedValue tv2 : cTV) {
			if (tv2.GetName().equalsIgnoreCase(n)) {
				tv = tv2;
				break;
			}
		}
		if (tv==null) {
			tv = cTV.AddNew(n, v);
	        if (!tv.Update()) {
	            result.addError("EA-Fehler: "+tv.GetLastError());
	        }
	        cTV.Refresh();
		} 
		else {
			String v2 = tv.GetValue();
			if (v2.equals(v)) {
			} else {
				tv.SetValue(v);
		        if (!tv.Update()) {
		            result.addError("EA-Fehler: "+tv.GetLastError());
		        }
			}
		}		
	}

	private void setTaggedValueRole(Collection<org.sparx.RoleTag> cTV, String n, String v) {
		org.sparx.RoleTag tv=null; 
		Iterator<RoleTag> itr =	cTV.iterator(); 
        while(itr.hasNext()) {
        	org.sparx.RoleTag tv2 = itr.next();
    		if (tv2.GetTag().equals(n)) {
    			tv = tv2;
    			break;
    		}
        }
		if (tv==null) {
			tv = cTV.AddNew(n, v);
	        if (!tv.Update()) {
        		result.addError("Fehler beim Setzen von TaggedValue '"+n+"'-'"+v+"': "+tv.GetLastError());
	        }
	        cTV.Refresh();
		} 
		else {
			String v2 = tv.GetValue();
			if (v2.equals(v)) {
        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"' (bestehender Wert)");
			} else {
				tv.SetValue(v);
		        if (!tv.Update()) {
	        		result.addError("Fehler beim Setzen von TaggedValue '"+n+"'-'"+v+"'/'"+v2+"': "+tv.GetLastError());
		        } else {
	        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"' (alter Wert: '"+v2+"')");		        	
		        }
			}
		}		
	}
	
	private void setTaggedValueAssociation(Collection<ConnectorTag> cTV, String n, String v) {
		ConnectorTag tv=null; 
		Iterator<ConnectorTag> itr = cTV.iterator(); 
        while(itr.hasNext()) {
        	org.sparx.ConnectorTag tv2 = itr.next();
    		if (tv2.GetName().equals(n)) {
    			tv = tv2;
    			break;
    		}
        }
		if (tv==null) {
			tv = cTV.AddNew(n, v);
	        if (!tv.Update()) {
        		result.addError("Fehler beim Setzen von TaggedValue '"+n+"'-'"+v+"': "+tv.GetLastError());
	        } else {
        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"'");
	        }
	        cTV.Refresh();
		} 
		else {
			String v2 = tv.GetValue();
			if (v2.equals(v)) {
        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"' (bestehender Wert)");
			} else {
				tv.SetValue(v);
		        if (!tv.Update()) {
	        		result.addError("Fehler beim Setzen von TaggedValue '"+n+"'-'"+v+"'/'"+v2+"': "+tv.GetLastError());
		        } else {
	        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"' (alter Wert: '"+v2+"')");		        	
		        }
			}
		}		
	}
	
	private void setTaggedValueAtt(Collection<AttributeTag> cTV, String n, String v) {
		AttributeTag tv=null; 
		Iterator<AttributeTag> itr = cTV.iterator(); 
        while(itr.hasNext()) {
        	AttributeTag tv2 = itr.next();
    		if (tv2.GetName().equals(n)) {
    			tv = tv2;
    			break;
    		}
        }
		if (tv==null) {
			tv = cTV.AddNew(n, v);
	        if (!tv.Update()) {
        		result.addError("Fehler beim Setzen von TaggedValue '"+n+"'-'"+v+"': "+tv.GetLastError());
	        } else {
        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"'");
	        }
	        cTV.Refresh();
		} 
		else {
			String v2 = tv.GetValue();
			if (v2.equals(v)) {
        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"' (bestehender Wert)");
			} else {
				tv.SetValue(v);
		        if (!tv.Update()) {
	        		result.addError("Fehler beim Setzen von TaggedValue '"+n+"'-'"+v+"'/'"+v2+"': "+tv.GetLastError());
		        } else {
	        		result.addDebug("Setzen von TaggedValue '"+n+"'-'"+v+"' (alter Wert: '"+v2+"')");		        	
		        }
			}
		}		
	}
	
	
	
	
	private void cloneStandardItems(Connector e, Info i) {
        e.SetAlias(i.aliasName());    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetNotes(i.documentation());    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetStereotype(stereotypesCSV(i.stereotypes()));    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        if(i.taggedValues() != null) {
	        for (Entry<String,String> me : i.taggedValues().entrySet()) {
	        	setTaggedValueAssociation(e.GetTaggedValues(),me.getKey(),me.getValue());
	        }
        }
	}

	private void cloneStandardItems(ConnectorEnd e, PropertyInfo i) {
        e.SetRole(i.name());    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetAlias(i.aliasName());    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        Multiplicity m = i.cardinality();
        if (m.maxOccurs==Integer.MAX_VALUE)
            e.SetCardinality(m.minOccurs+"..*");
        else
            e.SetCardinality(m.minOccurs+".."+m.maxOccurs);
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetIsNavigable(i.isNavigable());
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetRoleNote(i.documentation());    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        e.SetStereotype(stereotypesCSV(i.stereotypes()));    
        if (!e.Update()) {
            result.addError("EA-Fehler: "+e.GetLastError());
        }
        for (Entry<String,String> me : i.taggedValues().entrySet()) {
        	setTaggedValueRole(e.GetTaggedValues(),me.getKey(),me.getValue());
        }
	}

	public void write() {
		// nothing to do, everything will be written in initialise() and writeAll()
	}

	public int getTargetID() {
		return TARGET_UmlModel;
	}

	public void writeAll(ShapeChangeResult r) {
		if (rep==null)
			return; // repository not initialised

		for (AssociationInfo ai : associations) {
			PropertyInfo propi1 = ai.end1();
			PropertyInfo propi2 = ai.end2();
			ClassInfo ci1 = propi1.inClass();
			ClassInfo ci2 = propi2.inClass();
			Element c1 = classes.get(ci1);
			Element c2 = classes.get(ci2);
			if (c1==null)
				result.addWarning("Association between "+ci1.name()+" - "+ci2.name()+" not set as the first class is not part of target model.");
			else if (c2==null)
				result.addWarning("Association between "+ci1.name()+" - "+ci2.name()+" not set as the second class is not part of target model.");
			else {
				Connector con = c1.GetConnectors().AddNew ("", "Association");
				con.SetSupplierID(c2.GetElementID());
		    	con.SetDirection("Bi-Directional");
	            if (!con.Update()) {
	                result.addError("EA-Fehler: "+con.GetLastError());
	            }
	            c1.GetConnectors().Refresh();
	            cloneStandardItems(con,ai);
	            cloneStandardItems(con.GetClientEnd(),propi2);
	            cloneStandardItems(con.GetSupplierEnd(),propi1);
			}
		}
		
		for (Entry<ClassInfo,ClassInfo> entry : generalisations.entrySet()) {
			Element c1 = classes.get(entry.getKey());
			Element c2 = classes.get(entry.getValue());
			if (c1==null)
				result.addWarning("Generalisation "+entry.getKey().name()+" - "+entry.getValue().name()+" not set as the first class is not part of target model.");
			else if (c2==null)
				result.addWarning("Generalisation "+entry.getKey().name()+" - "+entry.getValue().name()+" not set as the second class is not part of target model.");
			else {
				Connector con = c1.GetConnectors().AddNew("", "Generalization");
				con.SetSupplierID(c2.GetElementID());
	            if (!con.Update()) {
	                result.addError("EA-Fehler: "+con.GetLastError());
	            }
	            c1.GetConnectors().Refresh();
			}
		}
		
		rep.Compact();
		rep.CloseFile();
		rep.Exit();
	}

	public void reset() {
		initialised = false;
		outputFilename = null;
		rep = null; 
		pOut = null;
	}

}
